'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = applyMiddleware;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _compose = require('./compose');

var _compose2 = _interopRequireDefault(_compose);

var _composeMiddleware = require('./composeMiddleware');

var _composeMiddleware2 = _interopRequireDefault(_composeMiddleware);

/**
 * Creates a higher-order store that applies middleware to a store's dispatch.
 * Because middleware is potentially asynchronous, this should be the first
 * higher-order store in the composition chain.
 * @param {...Function} ...middlewares
 * @return {Function} A higher-order store
 */

function applyMiddleware() {
  for (var _len = arguments.length, middlewares = Array(_len), _key = 0; _key < _len; _key++) {
    middlewares[_key] = arguments[_key];
  }

  return function (next) {
    return function () {
      var store = next.apply(undefined, arguments);
      var middleware = _composeMiddleware2['default'].apply(undefined, middlewares);

      function dispatch(action) {
        var methods = {
          dispatch: dispatch,
          getState: store.getState
        };

        return _compose2['default'](middleware(methods), store.dispatch)(action);
      }

      return _extends({}, store, {
        dispatch: dispatch
      });
    };
  };
}

module.exports = exports['default'];